require 'vec3'
require 'object'
vertex = require 'vertex'
face = require 'face'
camera = require 'camera'

function copy(o)
local ot = type(o) local c if ot == 'table' then c = {}
for k, v in next, o, nil do c[copy(k)] = copy(v) end
setmetatable(c, copy(getmetatable(o))) else c = o
end return c
end


function love.load()
math.randomseed(os.time())

lg = love.graphics
lk = love.keyboard
love.window.setMode(0, 0)
local h  =lg.getHeight()
love.window.setMode(h,h)
width = lg.getWidth()/2
height = lg.getHeight()/2
love.mouse.setRelativeMode(true)
camera:load()

--[[
LOAD NEW OBJECTS BY NEWOBJ() FUNCTION, ONLY SUPPORTS .OBJ FILE
AVAILABLE FUNCTIONS:

OBJ:getDepth() | TO SORT OBJECTS IN WORLD
OBJ:update()
OBJ:draw()
OBJ:setColor(R,G,B,A)
OBJ:translate(vec3)
OBJ:rotate(vec3)
OBJ:scale(vec3)
vec3 = OBJ:getSize()

TO MODIFY CAMERA MOVEMENT REMOVE THE COMMENTS IN CAMERA.LUA FILE
]]

objects = {}

temple = newObj("temple.obj")
temple:setColor(1,1,1)
temple:rotate(vec3(0, math.rad(90), 0))
table.insert(objects, temple)

tree = newObj("tree.obj")
tree:setColor(1,1,1)
tree:scale(vec3(0.75, 0.75, 0.75))
tree:rotate(vec3(0, math.rad(90), 0))
tree:translate(vec3(60, 0, 0))
table.insert(objects, tree)

cube = newObj("cube.obj")
cube.name = "cube"
cube:setColor(1,1,1)
cube:scale(vec3(15,15,15))
cube:translate(-cube:getSize()/2)
cube.translation = vec3(3, -60, -10)
cube:translate(cube.translation)
table.insert(objects, cube)

text = newObj("text.obj")
text:setColor(1,1,1)
text:rotate(vec3(math.rad(90), 0, 0))
text:translate(vec3(-75, 0, -10))

table.insert(objects, text)

camera:translate(vec3(0, -5, -100))
camera:setVelocity(vec3(50, 50, 50))

showline = false
showline_c = true
end

function love.update(dt)
if lk.isDown("f") then
if showline_c then
if showline then showline = false else showline = true end
showline_c = false
end
end
if not lk.isDown("f") then
showline_c = true
end
for i,v in pairs(objects) do
if v.name == "cube" then
v:translate(-v.translation)
v:rotate(vec3(math.rad(1),math.rad(1),math.rad(1)))
v:translate(v.translation)
end
v:update()
end
local function s(a,b)
return a:getDepth() > b:getDepth()
end
table.sort(objects, s)
temple:getDepth()
camera:update()
end

function love.mousemoved(x, y, dx, dy)
camera:rotate(vec3(math.rad(dx), math.rad(dy), 0))
end

function love.draw()
for i,v in pairs(objects) do
v:draw()
end
lg.setColor(1,1,1)
local f = lg.newFont(40)
lg.setFont(f)
lg.print("press F to show lines")
end